﻿// analysis_cmdline_args.h
#ifndef ANALYSIS_CMDLINE_ARGS_INCLUDED
#define ANALYSIS_CMDLINE_ARGS_INCLUDED

#include <functional>
#include <string>
#include <vector>

namespace cmd_args {

    // コマンドライン解析の結果
    enum class AnalysisCmdlineArgsResult {
        correct,                    // 正しい指定
        correct_exit,               // 正しい指定だが、プログラムは終了させる
        error_args_not_enough,      // エラー。コマンドライン引数が不足
        error_invalid_option,       // エラー。不正なオプションを発見
        error_invalid_parameter,    // エラー。パラメータの指定が正しくない
    };

    // オプション発見時のコールバック関数型
    using option_callback_t = std::function<void(const std::string&)>;

    // オプション情報
    struct OptionInfo {
        std::string                 short_name;     // 短い名前（不要なら空文字列）
        std::string                 long_name;      // 長い名前（不要なら空文字列）
        option_callback_t           callback_func;  // オプション発見時にコールバックする関数
        bool                        has_param;      // パラメータの有無
    };



    // コマンドライン引数を std::vector<std::string> に格納して返す
    std::vector<std::string> argv_to_vector(int argc, char** argv);

    // ヘルプオプションを登録する
    void regist_help(const OptionInfo& info);

    // バージョン情報オプションを登録する
    void regist_version(const OptionInfo& info);

    // コマンドライン引数を解析する
    AnalysisCmdlineArgsResult analysis_cmdline_args(const std::vector<std::string>& cmdline_args, std::size_t option_start_index, const std::vector<OptionInfo>& options);

}

#endif